% Kompensation des Phasen- und Amplitudenfehlers durch die parasitäre
% Shuntinduktivität;
% digitaler Tiefpassfilter im Frequenzbereich
% nach Marc Hauck, Masterarbeit, 2015

% Fa Abtastfrequenz in Hz
% y in V
% R des Shunts, bei Parallelschaltung R _eines_ Shunts, bei R<0 keine
% L (Induktivität des Shunts) in H (heha 170110)
% Frequenzkorrektur
% cutoff_freq in Hz, Filter deaktivieren mit sehr hoher Frequenz
function y_corr = freq_correction(Fa, y, R, L, cutoff_freq)
	% Parameter des PD-Glieds / Messshunts
	T = L/R;                                    % Zeitkonstante

	% FFT vorbereiten
	Fn = Fa/2;                                  % Nyquist-Frequenz
	NP = length(y);                             % Anzahl der Messwerte (Zeitbereich)
	df = Fa/NP;                                 % Frequenzauflösung
	fplot = -Fn:df:Fn-df;                       % Frequenzvektor für grafische Darstellung
	
	% ermittle Spektrum für Signal mit und ohne Korrektur
	FFTY = fft(y,NP);                           % führe FFT durch

	% schneide hohe Frequenzen ab
	Flow = find(fplot<-cutoff_freq, 1, 'last'); % Frequenzen die weggeschnitten werden
	Fhigh = find(fplot>cutoff_freq, 1, 'first');
	FFTYc = fftshift(FFTY);                     % kopiere FFTY-Spektrum zu korrigiertem Spektrum
	FFTYc(1:Flow) = 0;                          % setzte Frequenzanteile größer als Flow zu 0
	FFTYc(Fhigh:end) = 0;

	% frequenzkorrigiertes Signal
	if R > 0
		% PD Glied
%		PD = R*(1 + 1j.*2.*pi.*fplot.*T);        % Übertragungsfunktion PD
		PD = 1 + 1j.*2.*pi.*fplot.*T;            % Übertragungsfunktion PD (ohne R)

		FFTYc = FFTYc ./ PD;          % Dekonvolution: Division des Frequenzgangs des Signals durch den Frequenzgang des PD-Glieds
	end

	% Rücktransformation und grafische Darstellung der beiden Signale mit und ohne Korrektur
	y_corr = real(ifft(ifftshift(FFTYc),NP));               % Rücktransformation des Signals mit Korrektur
